@@ -20,6 +20,7 @@
 
 import logging
 import os
+import time
 from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
 
 logger = logging.getLogger(__name__)
@@ -58,12 +59,24 @@ class NodeImageDefinitions:
         return self._context.base_url
 
     def node_definitions(self):
+        """
+        Returns all node definitions.
+
+        :return: list of node definitions
+        :rtype: List[Dict]
+        """
         url = self._base_url + "node_definitions/"
         response = self.session.get(url)
         response.raise_for_status()
         return response.json()
 
     def image_definitions(self):
+        """
+        Returns all image definitions.
+
+        :return: list of image definitions
+        :rtype: List[Dict]
+        """
         url = self._base_url + "image_definitions/"
         response = self.session.get(url)
         response.raise_for_status()
@@ -71,15 +84,16 @@ class NodeImageDefinitions:
 
     def image_definitions_for_node_definition(self, definition_id):
         """
-        Returns the image definition for a given node definition
+        Returns all image definitions for a given node definition
 
         example::
 
             client_library.definitions.image_definitions_for_node_definition("iosv")
 
-        :param definition_id:
-        :returns: the image definition as JSON
-        :rtype: str
+        :param definition_id: node definition id
+        :type definition_id: str
+        :returns: list of image definition objects
+        :rtype: List[Dict]
         """
         url = (
             self._base_url + "node_definitions/" + definition_id + "/image_definitions"
@@ -88,15 +102,43 @@ class NodeImageDefinitions:
         response.raise_for_status()
         return response.json()
 
-    def upload_node_definition(self, body):
+    def upload_node_definition(self, body, json=False):
+        """
+        Upload new node definition.
+
+        :param body: node definition (yaml or json)
+        :type: str or dict
+        :param json: whether we are sending json data
+        :type json: bool
+        :return: "Success"
+        :rtype: str
+        """
         url = self._base_url + "node_definitions/"
-        response = self.session.post(url, data=body)
+        if json:
+            response = self.session.post(url, json=body)
+        else:
+            # YAML
+            response = self.session.post(url, data=body)
         response.raise_for_status()
         return response.json()
 
-    def upload_image_definition(self, body):
+    def upload_image_definition(self, body, json=False):
+        """
+        Upload new image definition.
+
+        :param body: image definition (yaml or json)
+        :type: str or dict
+        :param json: whether we are sending json data
+        :type json: bool
+        :return: "Success"
+        :rtype: str
+        """
         url = self._base_url + "image_definitions/"
-        response = self.session.post(url, data=body)
+        if json:
+            response = self.session.post(url, json=body)
+        else:
+            # YAML
+            response = self.session.post(url, data=body)
         response.raise_for_status()
         return response.json()
 
@@ -108,9 +150,9 @@ class NodeImageDefinitions:
 
             client_library.definitions.download_node_definition("iosv")
 
-        :param definition_id:
+        :param definition_id: the node definition ID
         :type definition_id: str
-        :returns: the node definition as JSON
+        :returns: the node definition as YAML
         :rtype: str
         """
         url = self._base_url + "node_definitions/" + definition_id
@@ -124,9 +166,9 @@ class NodeImageDefinitions:
 
             client_library.definitions.download_image_definition("iosv-158-3")
 
-        :param definition_id: the node definition ID
+        :param definition_id: the image definition ID
         :type definition_id: str
-        :returns: the image definition as JSON
+        :returns: the image definition as YAML
         :rtype: str
         """
 
@@ -182,6 +224,8 @@ class NodeImageDefinitions:
 
         :param definition_id: the definition ID to delete
         :type definition_id: str
+        :returns: "Success"
+        :rtype: str
         """
 
         url = self._base_url + "node_definitions/" + definition_id
@@ -199,6 +243,8 @@ class NodeImageDefinitions:
 
         :param definition_id: the image definition ID to remove
         :type definition_id: str
+        :returns: "Success"
+        :rtype: str
         """
 
         url = self._base_url + "image_definitions/" + definition_id
@@ -206,50 +252,28 @@ class NodeImageDefinitions:
         response.raise_for_status()
         return response.json()
 
-    def create_image_definition(self, image_id, node_definition_id, label, disk_image):
-        """
-        Creates a new image definition.
-
-        Example::
-
-            client_library.definitions.create_image_definition("test-1", "iosv", "test", "test.img")
-
-        :param image_id: the ID of the new image definition
-        :type image_id: str
-        :param node_definition_id: the node definition ID for this image definition
-        :type node_definition_id: str
-        :param label: the label of the new image definition
-        :type label: str
-        :param disk_image: the name of the .qcow2 disk image (must exist on server)
-        :type disk_image: str
-        """
-        body = {
-            "id": image_id,
-            "node_definition_id": node_definition_id,
-            "label": label,
-            "disk_image": disk_image,
-        }
-
-        params = {"json": True}
-
-        url = self._base_url + "image_definitions/" + image_id
-        response = self.session.post(url, json=body, params=params)
-        response.raise_for_status()
-        return response.json()
-
 
 def progress_callback(monitor):
+    if not hasattr(monitor, "start_time"):
+        monitor.start_time = time.time()
+    if not hasattr(monitor, "last_bytes"):
+        monitor.last_bytes = -1
     # Track progress in the monitor instance itself.
-    if not hasattr(monitor, "curr_progress"):
-        monitor.curr_progress = -1
+    if monitor.bytes_read > monitor.last_bytes:
+        print_progress_bar(monitor.bytes_read, monitor.len, monitor.start_time)
+        monitor.last_bytes = monitor.bytes_read
 
-    progress = int(100 * (monitor.bytes_read / monitor.len))
-    progress = min(progress, 100)
-    # Report progress every increment of 10%
-    if progress > monitor.curr_progress and progress % 10 == 0:
-        print(
-            "Progress: {0} of {1} bytes ({2}%)".format(
-                monitor.bytes_read, monitor.len, progress
-            )
-        )
-        monitor.curr_progress = progress
+
+def print_progress_bar(cur, total, start_time, length=50):
+    percent = ("{0:.1f}").format(100 * (cur / float(total)))
+    filled_len = int(round(length * cur / float(total)))
+    bar = "#" * filled_len + "-" * (length - filled_len)
+    raw_elapsed = time.time() - start_time
+    elapsed = time.strftime("[%H:%M:%S]", time.gmtime(raw_elapsed))
+    print(
+        "\r |{}| {}/{} {}% {}".format(bar, cur, total, percent, elapsed),
+        end="",
+        flush=True,
+    )
+    if cur == total:
+        print()
